@@ -22,8 +22,8 @@
 /** \file svg.h
  * \brief #GNU_gama::local::GamaLocalSVG class header file
  *
- * \author Ales Cepek 2012
- * \author Maxime Le Moual 2014
+ * \author Ales Cepek
+ * \author Maxime Le Moual
  */
 
 #ifndef GAMA_LOCAL_SVG_Gama_Local_Svg_gama_local_svg_h
@@ -45,7 +45,7 @@ namespace GNU_gama { namespace local {
       /** \param lnet pointer to local network object */
       GamaLocalSVG(LocalNetwork* lnet);
 
-      /** Returns SVG image as a std::string. */
+      /** Return SVG image as a std::string. */
       std::string string() const;
       /** Writes SVG image on to a standard stream */
       void draw(std::ostream& output_stream) const;
@@ -152,19 +152,26 @@ namespace GNU_gama { namespace local {
       typedef std::map<GNU_gama::local::PointID, shift> Shift;
       Shift shifts;
 
+      /** Shift scale. */
+      double shiftScale() const { return shiftscale; }
+
+      /** Set shift scale. */
+      void setShiftScale(double p) const { shiftscale = p; /*tst_implicit_size = false;*/ }
+
     private:
       LocalNetwork&          IS;
       const PointData&       PD;
       const ObservationData& OD;
+      const double y_sign;   // consistent coordinates +1, inconsistent -1
 
       mutable std::ostream*  svg;
 
-      mutable double T11, T12, T21, T22, Tx, Ty, Txsign, Tysign;
+      mutable double T11, T12, T21, T22, Tx, Ty;
 
-      // SVG coordinates bounding box and offset
+      // SVG coordinates, bounding box and offset
       mutable bool tst_implicit_size;
-      mutable double  minx, maxx, miny, maxy, offset;
-      mutable double ab_median, minimalsize, ellipsescale;
+      mutable double minx, maxx, miny, maxy, offset, extension;
+      mutable double ab_median, minimalsize, ellipsescale, ds_median, shiftscale;
       void svg_xy(const LocalPoint& point, double& x, double& y) const;
       void svg_draw_point  (const PointID& pid, const LocalPoint& point) const;
       void svg_point_shape (double x, double y,
@@ -175,12 +182,13 @@ namespace GNU_gama { namespace local {
       void svg_points      () const;
       void svg_observations() const;
       void svg_ellipse(const PointID& pid, double& a, double &b, double& alpha) const;
+      void svg_ellipse_bounding_box(double alpha, double a, double b, double& dx, double& dy) const;
 
       mutable double fontsize, symbolsize, strokewidth;
       mutable bool tst_draw_axes, tst_draw_point_symbols, tst_draw_point_ids,
           tst_draw_observations, tst_draw_ellipses, tst_draw_xy_shifts, tst_draw_z_shifts;
       mutable std::string  fixedsymbol, fixedfill, constrainedsymbol,
-        constrainedfill, freesymbol, freefill, xyshiftcolor, zshiftcolor;
+          constrainedfill, freesymbol, freefill, xyshiftcolor, zshiftcolor;
 
 
       /* helper svg point class */
@@ -214,9 +222,6 @@ namespace GNU_gama { namespace local {
       {
         TX = Point(tr11, tr12);
         TY = Point(tr21, tr22);
-
-        T11 = tr11; T12 = tr12;
-        T21 = tr21; T22 = tr22;
       }
     };
 }}
