@@ -1,5 +1,5 @@
 /* matvec_test_003.cpp
-   Copyright (C) 2000, 2012, 2018  Ales Cepek <cepek@gnu.org>
+   Copyright (C) 2000, 2012, 2018, 2024  Ales Cepek <cepek@gnu.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,89 +18,82 @@
 
 #include <iostream>
 #include <iomanip>
-#include <cmath>
 #include <matvec/hilbert.h>
 
 int main()
 {
-  typedef long double Double;
-
   using namespace std;
   using namespace GNU_gama;
 
-  int result = 0;
-  Double tol = 0;
-  
+  using Double = long double;
+  using Long = unsigned long;
+
   cout << "\n   Hilbert matrix  ....   test_003  matvec "
-       << GNU_gama::matvec_version() << "\n"
+       << GNU_gama::matvec_version() << "  inv()\n"
        << "------------------------------------------------------\n\n";
 
+  cout << "          cond         inv(H)        inv(I) \n";
+
   cout.setf(ios::scientific, ios::floatfield);
   cout.precision(2);
 
-  for (int N=2; N<=15; N++)
+  for (Long N=1; N<=15; N++)
     {
       cout << setw(2) << N;
 
-      Mat<Double> E(N,N);
+      Mat<Double, Long> E(N,N);
       E.set_identity();
-      Mat<Double> H = Hilbert   <Double, int, Exception::matvec>(N);
-      Mat<Double> I = InvHilbert<Double, int, Exception::matvec>(N);
+
+      Mat<Double, Long> H = Hilbert<Double, Long>(N);
+      Mat<Double, Long> I = InvHilbert<Double, Long>(N);
 
       Double fh = 0, fi = 0;
-      for (int i=1; i<=N; i++)
-        for (int j=1; j<=N; j++)
+      for (Long i=1; i<=N; i++)
+        for (Long j=1; j<=N; j++)
           {
-            fh += H(i,j)*H(i,j);
-            fi += I(i,j)*I(i,j);
+            fh += H(i,j)*H(i,j);  // Frobenius norm ||H|| = sqrt(fh)
+            fi += I(i,j)*I(i,j);  //                ||I|| = sqrt(fi)
           }
+      // estimation of the condition number is ||H|| \times ||I||
       cout << setw(14) << sqrt(fh*fi);
 
 
       try {
         cout << setw(14);
-        Mat<Double> T = Hilbert<Double, int, Exception::matvec>(N);
-        T.invert(tol);
+        Mat<Double, Long> T = Hilbert<Double, Long>(N);
+        T.invert();               // possible exception
         T = H*T - E;
 
         Double m = 0;
-        for (int i=1; i<=N; i++)
-          for (int j=1; j<=N; j++)
-            if (std::abs(T(i,j)) > std::abs(m))
+        for (Long i=1; i<=N; i++)
+          for (Long j=1; j<=N; j++)
+            if (fabs(T(i,j)) > fabs(m))
               m = T(i,j);
 
         cout << m;
       }
       catch (Exception::matvec&) {
         cout << "failed";
-	result++;
       }
 
       try {
         cout << setw(14);
-        Mat<Double> T = InvHilbert<Double, int, Exception::matvec>(N);
-        T.invert(tol);
+        Mat<Double, Long> T = InvHilbert<Double, Long>(N);
+        T.invert();               // possible exception
         T = I*T - E;
 
         Double m = 0;
-        for (int i=1; i<=N; i++)
-          for (int j=1; j<=N; j++)
-            if (std::abs(T(i,j)) > std::abs(m))
+        for (Long i=1; i<=N; i++)
+          for (Long j=1; j<=N; j++)
+            if (fabs(T(i,j)) > fabs(m))
               m = T(i,j);
 
         cout << m;
       }
-      catch (Exception::matvec&) {
+      catch (const Exception::matvec&) {
         cout << "failed";
-	result++;
       }
 
       cout << endl;
     }
-
-  // cout.setf(ios::fixed, ios::floatfield);
-  // cout.precision(0);
-  // cout << setw(9) << InvHilbert<Double>(15);
-
-  return result;
 }
